﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace FaceMonkey
{
    /// <summary>
    /// Element framework, allows contents to be scaled and rotated.
    /// bubbles mouse events 
    /// </summary>
    public partial class Container : UserControl
    {
        private bool MouseDown;
        private bool MouseDownResizing;
        private bool MouseDownRotating;
        private bool Disabled;
        public Size size = new Size(0.0,0.0);
        public Point pointer = new Point(0.0, 0.0);
        public Size OriginalSize = new Size(0.0, 0.0);
        public Size ScalingFactor = new Size(0.0, 0.0);
        public Size ScaledSize = new Size(0.0, 0.0);
        public Double Angle = 0.0;
        public Double AngleOffset = 0.0;
        public Size CurrentSize = new Size(0.0, 0.0);
        // event declaration
        public delegate void MouseUp_EventHandler(object sender, MouseButtonEventArgs e);
        public delegate void MouseDrag_EventHandler(object sender, MouseEventArgs e);
        public delegate void MouseUpResize_EventHandler(object sender, MouseButtonEventArgs e);
        public delegate void MouseDragResize_EventHandler(object sender, MouseEventArgs e);
        public delegate void MouseUpRotate_EventHandler(object sender, MouseButtonEventArgs e);
        public delegate void MouseDragRotate_EventHandler(object sender, MouseEventArgs e);
        public delegate void MouseUpStamp_EventHandler(object sender, MouseButtonEventArgs e);
        public event MouseUp_EventHandler MouseUp_Event;
        public event MouseDrag_EventHandler MouseDrag_Event;
        public event MouseUpResize_EventHandler MouseUpResize_Event;
        public event MouseDragResize_EventHandler MouseDragResize_Event;
        public event MouseUpRotate_EventHandler MouseUpRotate_Event;
        public event MouseDragRotate_EventHandler MouseDragRotate_Event;
        public event MouseUpStamp_EventHandler MouseUpStamp_Event;

        public Container() {
            InitializeComponent();
        }

        /// <summary>
        /// add UserControl as child and add required transforms
        /// must have "LayoutRoot"
        /// </summary>
        /// <param name="UControl"></param>
        public Container(UserControl UControl) {
            InitializeComponent();
            Canvas CanvasScale = new Canvas();
            CanvasScale.SetValue(Canvas.NameProperty, "CanvasScale");
            ScaleTransform st = new ScaleTransform();
            st.ScaleX = 1.0;
            st.ScaleY = 1.0;
            ScalingFactor.Width = st.ScaleX;
            ScalingFactor.Height = st.ScaleY;
            st.SetValue(Canvas.NameProperty, "ContentScale");
            RotateTransform rt = new RotateTransform();
            rt.SetValue(Canvas.NameProperty, "ContentRotate");
            rt.Angle = Angle;
            TransformGroup tg = new TransformGroup();
            tg.Children.Add(st);
            //tg.Children.Add(rt);
            CanvasScale.RenderTransform = tg;
            ContainerRoot.RenderTransform = rt;
            CanvasScale.Children.Add(UControl);
            ContainerContent.Children.Add(CanvasScale);
            UpdateContainer();
            UpdateRotation(0);
        }

        /// <summary>
        /// force setup once control is loaded
        /// </summary>
        private void Contents_Loaded(object sender, RoutedEventArgs e) {
            UpdateContainer();
        }

        /// <summary>
        /// set values and positions
        /// </summary>
        public void UpdateContainer() {
            Canvas c = FindLayoutRoot();
            ContainerContent.Width = c.ActualWidth;
            ContainerContent.Height = c.ActualHeight;
            Spacer1.Height = ContainerContent.Height - 32;
            Spacer2.Height = ContainerContent.Height - 32;
            c.UpdateLayout();
            size.Width = c.ActualWidth; // this will be used for the container/border
            size.Height = c.ActualHeight;
            FindCanvasScale().Width = size.Width;
            FindCanvasScale().Height = size.Height;
            OriginalSize = size;    // this is used by the scaling calculations
            CurrentSize = OriginalSize;
            ScaledSize.Width = OriginalSize.Width * ScalingFactor.Width;
            ScaledSize.Height = OriginalSize.Height * ScalingFactor.Height;
        }

        // called from the scaling system
        public void UpdateContainer(Size NewSize,bool scaling) {
            if (NewSize.Width > 32.0) { // add symetry to match the minimum available on the Y axis
                ContainerContent.Width = NewSize.Width;
            }
            else NewSize.Width = ContainerContent.Width;
            if (NewSize.Height > 32.0) {    // ensure scaling does not reduce to less than the controls height
                ContainerContent.Height = NewSize.Height;
                if ((ContainerContent.Height - 32) > 0.0) Spacer1.Height = ContainerContent.Height - 32;
                if ((ContainerContent.Height - 32) > 0.0) Spacer2.Height = ContainerContent.Height - 32;
            }
            else NewSize.Height = ContainerContent.Height;
            if (scaling) {
                UpdateScale(NewSize);
            }
            ContainerContent.UpdateLayout();
        }

        /// <summary>
        /// calculate scale based on the new size in comparison to the original size, this stops lots of scaling calculations further upstream
        /// </summary>
        /// <param name="NewSize"></param>
        private void UpdateScale(Size NewSize) {
            ScaleTransform ContentScaler = FindContentScale();
            Size NewScalingFactor = new Size(0.0, 0.0);
            ContentScaler.CenterX = 0.5;
            ContentScaler.CenterY = 0.5;
            if (NewSize.Width >0.0) {
                NewScalingFactor.Width = ((NewSize.Width / size.Width)* ScalingFactor.Width);
                ContentScaler.ScaleX = NewScalingFactor.Width;
            }
            if (NewSize.Height > 0.0) {
                NewScalingFactor.Height = ((NewSize.Height / size.Height)*ScalingFactor.Height);
                ContentScaler.ScaleY = NewScalingFactor.Height;
            }
            CurrentSize = new Size(OriginalSize.Width * NewScalingFactor.Width, OriginalSize.Height * NewScalingFactor.Height);
        }

        /// <summary>
        /// recalculate center for rotation and apply new calculated angle
        /// </summary>
        /// <param name="NewAngle"></param>
        public void UpdateRotation(Double NewAngle) {
            RotateTransform rt = FindContentRotate();
            rt.Angle = Angle+-NewAngle;
            rt.CenterX = (ContainerContent.Width + 2 + 16 + 16) / 2; // 2x 1px border, 2x 16px controls
            rt.CenterY = (ContainerContent.Height + 2) / 2; // 2x 1px border
        }

        /// <summary>
        /// return a point that is adjust from imput for current rotation
        /// </summary>
        /// <param name="NRPoint"></param>
        /// <returns></returns>
        public Point AdjustForRotation(Point NRPoint) {
            RotateTransform a = FindContentRotate();
            if (NRPoint.X == 0) NRPoint.X = FindContentScale().ScaleX * ContainerContent.Width + 2 + 16 + 16;
            if (NRPoint.Y == 0) NRPoint.Y = FindContentScale().ScaleY * ContainerContent.Height + 2;
            Double Radians = a.Angle * Math.PI / 180;
            // x′ = x × cos(θ) + y × sin(θ)
            // y′ = x × sin(θ) - y × cos(θ)
            Point RPoint = new Point();
            RPoint.X = NRPoint.X * Math.Cos(Radians) + NRPoint.Y * Math.Sin(Radians);
            RPoint.Y = NRPoint.X * Math.Sin(Radians) - NRPoint.Y * Math.Cos(Radians);
            return (RPoint);
        }

        /// <summary>
        /// start dragging
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ContainerContent_MouseLeftButtonDown(object sender, MouseButtonEventArgs e) {
            if (Disabled) return;
            UIElement el = (UIElement)sender;
            el.CaptureMouse();
            MouseDown = true;
            pointer = e.GetPosition(this);
            ZUp();
        }

        /// <summary>
        /// finish dragging
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ContainerContent_MouseLeftButtonUp(object sender, MouseButtonEventArgs e) {
            if (Disabled) return;
            MouseDown = false;
            MouseUp_Event(this, null);
        }

        /// <summary>
        /// handle dragging
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ContainerContent_MouseMove(object sender, MouseEventArgs e) {
            if (Disabled) return;
            if (!MouseDown) return;
            MouseDrag_Event(this, e);
            UIElement el = (UIElement)sender;
        }


        private void ContainerContent_LostMouseCapture(object sender, MouseEventArgs e) {
            MouseDown = false;
        }

        /// <summary>
        /// hide control, wastefully but works
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void CloseButton_MouseLeftButtonUp(object sender, MouseButtonEventArgs e) {
            if (Disabled) return;
            this.Opacity = 0.0;
        }

        /// <summary>
        /// hide borders and bubble event
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void StampButton_MouseLeftButtonUp(object sender, MouseButtonEventArgs e) {
            if (Disabled) return;
            C1.BorderThickness = new Thickness(0.0);
            C2.Opacity = 0.0;
            C3.Opacity = 0.0;
            Disabled = true;
            MouseUpStamp_Event(this, e);
        }

        /// <summary>
        /// force a high Z-Index 
        /// </summary>
        public void ZUp() {
            int z = (int)this.GetValue(Canvas.ZIndexProperty);
            z = 20;
            this.SetValue(Canvas.ZIndexProperty, z);
        }

        /// <summary>
        /// force a lower z index
        /// </summary>
        public void ZDown() {
            this.SetValue(Canvas.ZIndexProperty, 10);
        }

        /// <summary>
        /// begin scaling
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ScaleButton_MouseLeftButtonDown(object sender, MouseButtonEventArgs e) {
            ScaleTransform ContentScaler = FindContentScale();
            if (Disabled) return;
            UIElement el = (UIElement)sender;
            el.CaptureMouse();
            MouseDownResizing = true;
            pointer = new Point(-1.0, -1.0);  // they are everywhere
            size.Width = ContainerContent.Width;
            size.Height = ContainerContent.Height;
            ScalingFactor.Width = ContentScaler.ScaleX;
            ScalingFactor.Height = ContentScaler.ScaleY;
        }

        private void ScaleButton_LostMouseCapture(object sender, MouseEventArgs e) {
            MouseDownResizing = false; 
        }

        /// <summary>
        /// in scaling
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ScaleButton_MouseMove(object sender, MouseEventArgs e) {
            if (Disabled) return;
            if (MouseDownResizing)
            MouseDragResize_Event(this, e);
        }

        /// <summary>
        /// finish scaling
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ScaleButton_MouseLeftButtonUp(object sender, MouseButtonEventArgs e) {
            if (Disabled) return;
            UIElement el = (UIElement)sender;
            el.ReleaseMouseCapture();
            MouseDownResizing = false;
            MouseUpResize_Event(this, e);
            ScaleTransform ContentScaler = FindContentScale();
        }

        /// <summary>
        /// begin rotation
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void RotateButton_MouseLeftButtonDown(object sender, MouseButtonEventArgs e) {
            if (Disabled) return;
            UIElement el = (UIElement)sender;
            el.CaptureMouse();
            MouseDownRotating = true;
            RotateTransform rt = FindContentRotate();
            AngleOffset = FindAngle();
            Angle = AngleOffset; // current angle from which to work from. 
        }

        /// <summary>
        /// find the angle to the rotate control (it used to be bottom right)
        /// </summary>
        /// <returns></returns>
        private double FindAngle() {
            Point ControlLoc = new Point(ContainerContent.Width+16+16+1, 0); // half a control+a control width+1px border
            Point CenterLoc = new Point();
            Size Dimension = new Size(ContainerContent.Width+16+16+2, ContainerContent.Height+2);
            Dimension.Width /= 2;
            Dimension.Height /= 2;
            CenterLoc.X = Dimension.Width;
            CenterLoc.Y = Dimension.Height;
            Double Radians = Math.Atan2(-ControlLoc.Y + CenterLoc.Y,ControlLoc.X - CenterLoc.X );
            Double NewAngle = Radians * (180 / Math.PI);
            NewAngle += 90;
            RotateTransform rt = FindContentRotate();
            return (NewAngle);
        }

        /// <summary>
        /// in rotation
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void RotateButton_MouseMove(object sender, MouseEventArgs e) {
            if (Disabled) return;
            if (MouseDownRotating)
               MouseDragRotate_Event(this, e);
        }

        /// <summary>
        /// finish rotating
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void RotateButton_MouseLeftButtonUp(object sender, MouseButtonEventArgs e) {
            if (Disabled) return;
            UIElement el = (UIElement)sender;
            el.ReleaseMouseCapture();
            MouseDownResizing = false;
            MouseUpRotate_Event(this, e);
        }

        private void RotateButton_LostMouseCapture(object sender, MouseEventArgs e) {
            MouseDownRotating = false;
        }

        /// <summary>
        /// short cut returning the child "LayoutRoot"
        /// </summary>
        /// <returns></returns>
        private Canvas FindLayoutRoot() {
            Canvas path = (Canvas)ContainerContent.Children[0];
            UserControl uc = (UserControl)path.Children[0];
            return( (Canvas)uc.FindName("LayoutRoot") );
        }

        /// <summary>
        /// shortcut returning the childs scaling canvas
        /// </summary>
        /// <returns></returns>
        private Canvas FindCanvasScale() {
            return( (Canvas)ContainerContent.Children[0] );
        }

        /// <summary>
        /// shortcut returning rotate transform, used to be part of the child
        /// </summary>
        /// <returns></returns>
        private RotateTransform FindContentRotate() {
            Canvas iR = FindCanvasScale();
            return ((RotateTransform)iR.FindName("ContentRotate"));
        }

        /// <summary>
        /// shortcut returning the childs content canvas
        /// </summary>
        /// <returns></returns>
        private ScaleTransform FindContentScale() {
            Canvas iR = FindCanvasScale();
            return ((ScaleTransform)iR.FindName("ContentScale"));
        }

    }
}
