﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.IO;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Ink;
using System.Windows.Threading;

// FaceMonkey Rob Davis 2008
// Simple fun drawing application
// silverlight 2 RTW
namespace FaceMonkey
{
    public partial class Page : UserControl
    {
        private Stroke InkStroke;
        private int BrushColourIndex;
        private int BrushSize = 2;
        private bool CaligBrush;    // state of the brush
        private DispatcherTimer InkTimer = new DispatcherTimer();   // animates the drawing title
        private int InkIndex; // drawing title coordinates
        private byte r, g, b; // drawing title stroke colours
        public Random random = new Random();

        public Page() {
            InitializeComponent();
            // start the title drawing
            InkTimer.Interval = new TimeSpan(0, 0, 0, 0, 10);
            InkTimer.Tick += new EventHandler(InkTimer_Tick);
            InkTimer.Start();
            AddButtons();
        }

 
        /// <summary>
        /// For each timer tick add a new stroke to the Ink Presenter with a colour
        /// </summary>
        void InkTimer_Tick(object sender, EventArgs e) {
            if (InkIndex < DrawFaceMonkey.data.Length) {
                if (InkStroke == null || (DrawFaceMonkey.data[InkIndex]==0 && DrawFaceMonkey.data[InkIndex+1]==0)) {
                    InkStroke = new Stroke();
                    InkStroke.DrawingAttributes.Height = 20;
                    InkPx.Strokes.Add(InkStroke);
                    byte[] by = new byte[3];
                    random.NextBytes(by);
                    r = by[0];
                    g = by[1];
                    b = by[2];
                }
                if (DrawFaceMonkey.data[InkIndex] != 0 && DrawFaceMonkey.data[InkIndex + 1] != 0) {
                    if (r < 255) r++;
                    else if (g < 255) g++;
                    else if (b < 255) b++;
                    if (r > 200) r = 200;
                    if (g > 200) g = 200; 
                    if (b > 200) b = 200;
                    InkStroke.DrawingAttributes.Color = Color.FromArgb(255, r, g, b);
                    InkStroke.StylusPoints.Add(new StylusPoint(DrawFaceMonkey.data[InkIndex], DrawFaceMonkey.data[InkIndex + 1]));
                }
                InkIndex += 2;
            }
            else {
                if (InkTimer.IsEnabled) InkTimer.Stop();
                InkStroke = null;
                InkIndex = -1;
            }
        }

        /// <summary>
        /// Remove all objects from the playground drawing area
        /// </summary>
        private void Clear() {
            InkPx.Strokes.Clear();
            Background.Children.Remove(Face);
            ULImage.Source = null;
            List<UIElement> objs = new List<UIElement>();
            foreach (UIElement obj in Playground.Children) {
                if ((obj is FaceMonkey.Container) || (obj is InkPresenter) || (obj is SwissCheese)) {
                    objs.Add(obj);
                }
            }
            foreach (UIElement obj in objs) {
                Playground.Children.Remove(obj);
            }
            BackgroundRotateTransform.Angle = 0;
            BackgroundScaleTransform.ScaleY = 1.0;
            BackgroundScaleTransform.ScaleX = 1.0;
            // add an inkpresenter back in
            NewInkPresenter();
        }

        // clear everything and allow user to select a new background image
        private void LoadImage_Click(object sender, RoutedEventArgs e) {
            Clear();
            ULImage.Source = LoadUtils.LoadNewImage();
            SizeImage.IsEnabled = true;
            ShowControls.IsEnabled = true;
            WipeClear.IsEnabled = true;
        }

        /// <summary>
        /// stretch background image
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void SizeImage_Click(object sender, RoutedEventArgs e) {
            Button button = (Button)sender;
            if (button.Content.ToString() == "Zoom background to Fit") {
                ULImage.Stretch = Stretch.Uniform;
                button.Content = "Set background to original Size";
            }
            else {
                ULImage.Stretch = Stretch.None;
                button.Content = "Zoom background to Fit";
            }
        }

        /// <summary>
        /// open instructions window
        /// </summary>
        private void Instructions_Click(object sender, RoutedEventArgs e) {
            InstructionsPage i = new InstructionsPage();
            i.Name = "instructionsp";
            i.SetValue(Canvas.ZIndexProperty, 9222);
            i.MouseUp_Event += new InstructionsPage.MouseUp_EventHandler(Instructions_MouseUp_Event);
            LayoutRoot.Children.Add(i);
            GridButton.Opacity = 0.0;
        }

        // remove Instructions window
        void Instructions_MouseUp_Event(object sender, MouseButtonEventArgs e) {
            LayoutRoot.Children.Remove((UIElement)LayoutRoot.FindName("instructionsp"));
            GridButton.Opacity = 1.0;
        }

        /// <summary>
        /// open about window
        /// </summary>
        private void About_Click(object sender, RoutedEventArgs e) {
            AboutPage i = new AboutPage();
            i.Name = "iAboutp";
            i.SetValue(Canvas.ZIndexProperty, 9222);
            i.MouseUp_Event += new AboutPage.MouseUp_EventHandler(iAbout_MouseUp_Event);
            LayoutRoot.Children.Add(i);
            GridButton.Opacity = 0.0;
        }

        /// <summary>
        /// remove about window
        /// </summary>
        void iAbout_MouseUp_Event(object sender, MouseButtonEventArgs e) {
            LayoutRoot.Children.Remove((UIElement)LayoutRoot.FindName("iAboutp"));
            GridButton.Opacity = 1.0;
        }

        // show/hide controls via short animation
        private void ShowControls_Click(object sender, RoutedEventArgs e) {
            Button b = (Button)sender;
            if (b.Content.ToString() == "Show Controls") {
                b.Content = "Hide Controls";
                ControlPanelIn.Begin();
            }
            else {
                b.Content = "Show Controls";
                ControlPanelOut.Begin();
            }
        }

        /// <summary>
        /// process ink presenter strokes
        /// </summary>
        void Ink_MouseMove(object sender, MouseEventArgs e) {
            if (InkIndex >-1) return;
            if (InkStroke != null) {
                InkStroke.StylusPoints.Add(new StylusPoint(e.GetPosition(Playground).X, e.GetPosition(Playground).Y));
            }
        }

        /// <summary>
        /// process ink presenter event
        /// </summary>
        void Ink_MouseLeftButtonUp(object sender, MouseButtonEventArgs e) {
            if (InkTimer.IsEnabled) return;
            InkStroke = null;
        }

        /// <summary>
        /// process ink presenter event and set stroke values, size color, shape
        /// </summary>
        void Ink_MouseLeftButtonDown(object sender, MouseButtonEventArgs e) {
            InkPresenter InkP = (InkPresenter)sender;
            if (InkIndex > -1) return;
            InkStroke = new Stroke();
            InkStroke.StylusPoints.Add(new StylusPoint(e.GetPosition(Playground).X, e.GetPosition(Playground).Y));
            switch (BrushColourIndex) {
                case 1:
                    InkStroke.DrawingAttributes.Color = Colors.Red;
                    break;
                case 2:
                    InkStroke.DrawingAttributes.Color = Colors.Blue;
                    break;
                case 3:
                    InkStroke.DrawingAttributes.Color = Colors.Green;
                    break;
                default:
                    InkStroke.DrawingAttributes.Color = Colors.Black;
                    break;
            }
            switch (BrushSize) {
                case 1:
                    InkStroke.DrawingAttributes.Height = 5;
                    if (CaligBrush) InkStroke.DrawingAttributes.Width = InkStroke.DrawingAttributes.Height;
                    break;
                case 2:
                    InkStroke.DrawingAttributes.Height = 10;
                    if (CaligBrush) InkStroke.DrawingAttributes.Width = InkStroke.DrawingAttributes.Height;
                    break;
                case 3:
                    InkStroke.DrawingAttributes.Height = 20;
                    if (CaligBrush) InkStroke.DrawingAttributes.Width = InkStroke.DrawingAttributes.Height;
                    break;
                default:
                    InkStroke.DrawingAttributes.Height = 1;
                    if (CaligBrush) InkStroke.DrawingAttributes.Width = InkStroke.DrawingAttributes.Height;
                    break;
            }
            InkP.Strokes.Add(InkStroke);
        }

        // not used
        private void Playground_MouseLeftButtonUp(object sender, MouseButtonEventArgs e) {        }
        // not used
        private void Playground_MouseLeftButtonDown(object sender, MouseButtonEventArgs e) {        }

        // readability enhancer :)
        private void SetBrushColour(int Index) {
            BrushColourIndex = Index;
        }

        // readability enhancer :)
        private void SetBrushSize(int Size) {
            BrushSize = Size;
        }

        /// <summary>
        /// Process each buttons click using the name to route actions
        /// note to self, next time use more meaningful names
        /// </summary>
        private void AddButton_Click(object sender, RoutedEventArgs e) {
            Rectangle button = (Rectangle)sender;
            Container c=null;
            switch (button.Name) {
                case "H1": c = new Container(new hat_1());
                    break;
                case "H2": c = new Container(new hat_2());
                    break;
                case "H3": c = new Container(new hat_3());
                    break;
                case "H4": c = new Container(new hat_4());
                    break;
                case "G1": c = new Container(new glasses_1());
                    break;
                case "G2": c = new Container(new glasses_2());
                    break;
                case "G3": c = new Container(new glasses_3());
                    break;
                case "G4": c = new Container(new glasses_4());
                    break;
                case "E1": c = new Container(new eye_1());
                    break;
                case "E2": c = new Container(new eye_2());
                    break;
                case "E3": c = new Container(new eye_3());
                    break;
                case "E4": c = new Container(new eye_4());
                    break;
                case "R1": c = new Container(new ear_1());
                    break;
                case "R2": c = new Container(new ear_2());
                    break;
                case "R3": c = new Container(new ear_3());
                    break;
                case "R4": c = new Container(new ear_4());
                    break;
                case "N1": c = new Container(new nose_1());
                    break;
                case "N2": c = new Container(new nose_2());
                    break;
                case "N3": c = new Container(new nose_3());
                    break;
                case "N4": c = new Container(new nose_4());
                    break;
                case "M1": c = new Container(new mouth_1());
                    break;
                case "M2": c = new Container(new mouth_2());
                    break;
                case "M3": c = new Container(new mouth_3());
                    break;
                case "M4": c = new Container(new mouth_4());
                    break;
                case "O1": c = new Container(new other_1());
                    break;
                case "O2": c = new Container(new other_2());
                    break;
                case "O3": c = new Container(new other_3());
                    break;
                case "O4": c = new Container(new facemonkey());
                    break;
                default:
                    break;
            }
            char Type = button.Name.ToCharArray()[0];
            char Value = button.Name.ToCharArray()[1];
            if (Type == 'S') SetBrushSize(Convert.ToInt32(Value.ToString()));
            if (Type == 'C') SetBrushColour(Convert.ToInt32(Value.ToString()));
            if (Type == 'Q' && Value == '0') CaligBrush = !CaligBrush;
            if (Type == 'Q' && (Value == '1' || Value == '2')) {
                RotateBackground(Value == '2');
            }
            if (Type == 'Q' && Value == '3') c = new Container(new ImageContainer());
            if (Type == 'Q' && Value == '4') {
                VideoContainer vc = new VideoContainer();
                LoadUtils.LoadNewZideo(vc.video);
                c = new Container(vc);
            }
            if (Type == 'Q' && (Value == '5' || Value == '6')) {
                Mirror(Value == '5');
            }
            if (Type == 'Q' && Value == '7') {
                SwissCheese sw = new SwissCheese();
                sw.SetValue(Canvas.ZIndexProperty, 10);
                sw.Completed_Event += new EventHandler(sw_Completed_Event);
                Playground.Children.Add(sw);
                NewInkPresenter();
            }
            if (c != null) {
                // add events and properties to all items that are contained within Containers
                c.SetValue(Canvas.ZIndexProperty, 20);
                c.MouseDrag_Event += new Container.MouseDrag_EventHandler(Container_MouseDrag_Event);
                c.MouseUp_Event += new Container.MouseUp_EventHandler(Container_MouseUp_Event);
                c.MouseDragResize_Event += new Container.MouseDragResize_EventHandler(c_MouseDragResize_Event);
                c.MouseUpResize_Event += new Container.MouseUpResize_EventHandler(c_MouseUpResize_Event);
                c.MouseDragRotate_Event += new Container.MouseDragRotate_EventHandler(c_MouseDragRotate_Event);
                c.MouseUpRotate_Event += new Container.MouseUpRotate_EventHandler(c_MouseUpRotate_Event);
                c.MouseUpStamp_Event += new Container.MouseUpStamp_EventHandler(c_MouseUpStamp_Event);
                Playground.Children.Add(c);
            } 
        }

        // not used
        void sw_Completed_Event(object sender, EventArgs e) {        }

        /// <summary>
        /// Rotate the background image or monkey
        /// compensates for scaling
        /// </summary>
        /// <param name="CCW">clockwise true/false</param>
        private void RotateBackground(bool CCW) {
            if ( 
                ((BackgroundScaleTransform.ScaleY == -1) && (BackgroundScaleTransform.ScaleX == 1))  ||
                ((BackgroundScaleTransform.ScaleY == 1) && (BackgroundScaleTransform.ScaleX == -1))
                ) CCW = !CCW;
            if (CCW) BackgroundRotateTransformDA.By = 90.0;
            else BackgroundRotateTransformDA.By = -90.0;
            if (BackgroundRotateTransform.Angle % 90.0 == 0) BackgroundRotateTransformAnim.Begin();

        }

        // flip the background image or monkey
        private void Mirror(bool mirror) {
            if (mirror) {
                if (BackgroundScaleTransform.ScaleY < 1) {
                    BackgroundScaleTransformDAY.From = -1.0;
                    BackgroundScaleTransformDAY.To = 1.0;
                }
                else {
                    BackgroundScaleTransformDAY.From = 1.0;
                    BackgroundScaleTransformDAY.To = -1.0;
                }
                BackgroundScaleTransformAnimY.Begin();
            }
            else {
                if (BackgroundScaleTransform.ScaleX < 1) {
                    BackgroundScaleTransformDAX.From = -1.0;
                    BackgroundScaleTransformDAX.To = 1.0;
                }
                else {
                    BackgroundScaleTransformDAX.From = 1.0;
                    BackgroundScaleTransformDAX.To = -1.0;
                }
                BackgroundScaleTransformAnimX.Begin();
            }
        }

        /// <summary>
        /// add a new ink presenter to the playground the moment an element is stamped
        /// </summary>
        void c_MouseUpStamp_Event(object sender, MouseButtonEventArgs e) {
            Container c = (Container)sender;
            c.SetValue(Canvas.ZIndexProperty, 10);
            NewInkPresenter();
        }

        /// <summary>
        /// add a new ink presenter to the playground 
        /// </summary>
        void NewInkPresenter() {
            InkPresenter iP = new InkPresenter();
            iP.MouseLeftButtonDown += new MouseButtonEventHandler(Ink_MouseLeftButtonDown);
            iP.MouseLeftButtonUp += new MouseButtonEventHandler(Ink_MouseLeftButtonUp);
            iP.MouseMove += new MouseEventHandler(Ink_MouseMove);
            iP.SetValue(Canvas.ZIndexProperty, 10);
            iP.Width = 800;
            iP.Height = 580;
            iP.Background = new SolidColorBrush(Colors.Transparent);
            Opacity = 1.0;
            Playground.Children.Add(iP);
        }

        // not used
        void c_MouseUpRotate_Event(object sender, MouseButtonEventArgs e) {           }

        /// <summary>
        /// Update the Container with a new angle based on current mouse position to avoid mismatches
        /// </summary>
        void c_MouseDragRotate_Event(object sender, MouseEventArgs e) {
            Container c =  (Container)sender;
            Point MouseLoc = e.GetPosition(this);
            Size Dimension = c.CurrentSize;
            Dimension.Width /= 2;
            Dimension.Height /= 2;
            Dimension.Width += 16 + 1; // adjust size for left hand control, make sense
            Dimension.Height += 16 + 1;  // no idea why this is needed but it works </shrug>
            Point CenterLoc = new Point(
                (Double)c.GetValue(Canvas.LeftProperty),
                (Double)c.GetValue(Canvas.TopProperty) 
                );
            CenterLoc.X += Dimension.Width;
            CenterLoc.Y += Dimension.Height ;
            Double Radians = Math.Atan2(MouseLoc.X - CenterLoc.X, MouseLoc.Y - CenterLoc.Y);
            Double NewAngle = Radians * (180 / Math.PI);
            c.UpdateRotation(NewAngle);
        }

        // not used
        void c_MouseUpResize_Event(object sender, MouseButtonEventArgs e) {        }

        /// <summary>
        /// scaling consistently
        /// </summary>
        void Container_ScaleProcessor(Container c, Point Current, Point p) {
            if ((c.pointer.X == -1.0 && c.pointer.Y == -1.0) ) c.pointer = Current; // more magic
            double OriginalSizeY = c.size.Height;
            double OnePercentY = OriginalSizeY / 100;
            Size NewSize = new Size();
            bool PositiveY;
            double DiffY;
            if (Current.Y < c.pointer.Y) {
                PositiveY = false;
                DiffY = c.pointer.Y - Current.Y;
            }
            else {
                PositiveY = true;
                DiffY = Current.Y - c.pointer.Y;
            }
            if (DiffY > 0) {
                if (!PositiveY) {
                    NewSize.Height = ((OriginalSizeY + DiffY)>0.0)?(OriginalSizeY + DiffY):0;
                }
                else {
                    NewSize.Height = ((OriginalSizeY - DiffY)>0.0)?(OriginalSizeY - DiffY):0;
                }
            }
            double OriginalSizeX = c.size.Width;
            double OnePercentX = OriginalSizeX / 100;
            bool PositiveX;
            double DiffX;
            if (Current.X < c.pointer.X) {
                PositiveX = false;
                DiffX = c.pointer.X - Current.X;
            }
            else {
                PositiveX = true;
                DiffX = Current.X - c.pointer.X;
            }
            if (DiffX > 0) {
                if (!PositiveX) {
                    NewSize.Width = ((OriginalSizeX - DiffX) > 0.0) ? (OriginalSizeX - DiffX) : 0;
                }
                else {
                    NewSize.Width = ((OriginalSizeX + DiffX)>0.0)?(OriginalSizeX + DiffX):0;
                }
            }
            if (NewSize.Height>32) { // do not resize if box is to small
               // c.SetValue(Canvas.TopProperty, (Current.Y) - 30); // keep resize button under mouse cursor, looking for more elegant solution
            }
            c.UpdateContainer(NewSize, true);
        }

        /// <summary>
        /// Before passing drag event mouse coords to scaling compensate for any current rotation
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void c_MouseDragResize_Event(object sender, MouseEventArgs e) {
            Point RPoint = ((Container)sender).AdjustForRotation(e.GetPosition(this));
            Container_ScaleProcessor((Container)sender, RPoint, e.GetPosition(((Container)sender).ScaleButton));
        }

        // not used
        void Container_MouseUp_Event(object sender, MouseButtonEventArgs e) {  }

        /// <summary>
        /// relocate element based on mouse position
        /// </summary>
        void Container_MouseDrag_Event(object sender, MouseEventArgs e) {
            Container c = (Container)sender;
            double X, Y;
            X = (e.GetPosition(this).X - c.pointer.X);
            Y = (e.GetPosition(this).Y -c.pointer.Y) -20;
            c.SetValue(Canvas.LeftProperty, X);
            c.SetValue(Canvas.TopProperty, Y);
        }

        /// <summary>
        /// initiate clearing
        /// </summary>
        private void WipeClear_Click(object sender, RoutedEventArgs e) {
            Clear();
        }
    }
}
