﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace FaceMonkey
{
    /// <summary>
    /// very similar to the other_3 class
    /// create random animated elements
    /// </summary>
    public partial class SwissCheese : UserControl
    {
        private Random random = new Random();
        private DispatcherTimer timer = new DispatcherTimer();
        private int count;
        public event EventHandler Completed_Event;

        public SwissCheese() {
            InitializeComponent();
            createRandomHole(); // kick off
        }

        /// <summary>
        /// Creates a random 'hole' ellipse and corosponding animation
        /// </summary>
        private void createRandomHole() {
            Point point = new Point();
            point.X = random.Next((int)LayoutCheese.Width);
            point.Y = random.Next((int)LayoutCheese.Height);
            timer = new DispatcherTimer();
            timer.Interval = new TimeSpan(0, 0, 0, 0, random.Next(500));
            timer.Tick += delegate
            {
                addHole(point, false);   
            };
            timer.Start();
        }
                /// <summary>
        /// Creates an ellipse, three animations and the storyboard to hold them all and then executes it.
        /// </summary>
        /// <param name="point">Point where ellipse should be created</param>
        /// <param name="ui">Flag whether or not this call was user instagated</param>
        private void addHole(Point point, bool ui)
        {
            if (count > 40) {
                timer.Stop();
                Completed_Event(this, EventArgs.Empty); 
                return;
            }
            if (!ui) timer.Stop();  // if random stop the timer and wait for the call at the end of this function to kick off a new random blip
            Storyboard storyboard = new Storyboard();
            storyboard.Completed += new EventHandler(stb_Completed); // kills the storyboard once it has completed
            DoubleAnimation doubleanimationscaleX = new DoubleAnimation();
            DoubleAnimation doubleanimationscaleY = new DoubleAnimation();
            DoubleAnimation doubleaanimationopacity = new DoubleAnimation();
            ScaleTransform scaletransform = new ScaleTransform();
            Ellipse ellipse = new Ellipse();
            ellipse.Width = 20 + random.Next(20);
            ellipse.Height = ellipse.Width;
            ellipse.Fill = new SolidColorBrush(Colors.White);
            ellipse.StrokeThickness = 0;
            ellipse.SetValue(Canvas.LeftProperty, point.X);
            ellipse.SetValue(Canvas.TopProperty, point.Y);
            ellipse.SetValue(Ellipse.NameProperty, string.Format("blip{0}", count));
            scaletransform.CenterX = 20;
            scaletransform.CenterY = 20;
            scaletransform.ScaleX = 1;
            scaletransform.ScaleY = 1;
            ellipse.RenderTransform = scaletransform; // before being animatable transform must exist
            LayoutCheese.Children.Add(ellipse);
            doubleanimationscaleX.From = 1;
            doubleanimationscaleX.To = 5;
            doubleanimationscaleX.Duration = new Duration(new TimeSpan(0, 0, 0, 1));
            doubleanimationscaleY.From = 1;
            doubleanimationscaleY.To = 6;
            doubleanimationscaleY.Duration = new Duration(new TimeSpan(0, 0, 0, 1));
            doubleaanimationopacity.From = 0;
            doubleaanimationopacity.To = 1;
            doubleaanimationopacity.Duration = new Duration(new TimeSpan(0, 0, 0, 0,500));
            Storyboard.SetTarget(doubleanimationscaleX, scaletransform);
            Storyboard.SetTargetProperty(doubleanimationscaleX, new PropertyPath("(ScaleX)"));
            Storyboard.SetTarget(doubleanimationscaleY, scaletransform);
            Storyboard.SetTargetProperty(doubleanimationscaleY, new PropertyPath("(ScaleY)"));
            Storyboard.SetTarget(doubleaanimationopacity, ellipse);
            Storyboard.SetTargetProperty(doubleaanimationopacity, new PropertyPath("(Opacity)"));
            storyboard.Children.Add(doubleanimationscaleX);
            storyboard.Children.Add(doubleanimationscaleY);
            storyboard.Children.Add(doubleaanimationopacity);
            LayoutCheese.Resources.Add(String.Format("stroyboard{0}", count), storyboard); // with all the pieces in place, add the storyboard to the canvas
            count++;
            storyboard.Begin();
            if (!ui) createRandomHole(); // as this was not mouse related start a new random blip
        }

        /// <summary>
        /// deletes the storyboard when its finished
        /// the ellipse are never removed :(
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void stb_Completed(object sender, EventArgs e)
        {
            Storyboard storyboard = (Storyboard)sender;
            String name = (string)storyboard.GetValue(Canvas.NameProperty);
            if (name != null) LayoutCheese.Resources.Remove(name);
        }

    }
}
